-- Termites and Wood Chips
-- Authors: Colin Broderick & Stjepan Rajcic

-------------------------------------------
-- GLOBAL
-------------------------------------------

EMPTY = 0
FULL = 1
WOOD = 2

SEED = 10
rand = Random{}

-------------------------------------------
-- MODEL 
-------------------------------------------

--# space
cs = CellularSpace {
	xdim = 30,
	ydim = 30
}
--#creates neighborhood
cs:createNeighborhood{
	strategy = "moore", --vonneumann
		self = false}

--#places wood and empty cells
forEachCell(cs, function(cell)
	if rand:number() > 0.7 then
		cell.state = WOOD
		
	else
		cell.state = EMPTY
	end
end)


--# behavior 

singleFooAgent = Agent {

        -- at the beginig agent doesn't have a chip
	hasChip = 0,
	execute = function(self)
	
		-- random sample
		local cell = cs:sample()
		
		-- get the wood in the neighbourhood of the agent if iit exists
		local foundCell = schibe(self)
		
		-- check if the agent is not carrying a chip and that there is a wood in the neighbourhood of the agent
		if self.hasChip == 0 and foundCell then
		
			-- check if there is a random cell in the space where agent can move
			if cell.state == EMPTY then
			
			        -- neighhbouring cell becomes empty
			        moveMeHere = foundCell
			        moveMeHere.state = EMPTY
			        
			        --agent moves to the empty random cell in the space
					self:getCell().state = EMPTY
					self:move(cell)
					cell.state = FULL
					
					--agent has a chip
					self.hasChip = 1
				
			end
			
		    -- check if the agent is carrying a chip and that there is a wood in the neighbourhood of the agent
			elseif self.hasChip == 1 and foundCell then
				
				-- check if there is a random cell in the space where agent can move
				if cell.state == EMPTY then
				
				    --agent's cell becomes wood
				    self:getCell().state = WOOD
				    
				    --agent moves to the empty random cell in the space
					self:move(cell)
					cell.state = FULL
					
					--agent doesn't have a chip
					self.hasChip = 0
				end
						
			else
				if cell.state == EMPTY then
					self:getCell().state = EMPTY
					self:move(cell)
					cell.state = FULL
				end
			end
		
				
		
		end
	}
    -- get the wood in the neighbourhood of the agent if it exists
	function schibe (test)
			local cellnn = test:getCell()
			local found_cell = nil;
			forEachNeighbor (cellnn, function (c, neigh)
					if neigh.state == WOOD then
						found_cell = neigh
						return false
					end
			end)
			return found_cell
	end


--#society

soc = Society{
	instance = singleFooAgent,
	quantity = 50
}

--# time

t = Timer {
	Event{action = soc},
	Event{action = cs}
}

--# environment

e = Environment {
	cs,
	soc,
	t
}

e:createPlacement{strategy = "random", max = 1}

-------------------------------------------
-- SIMULATION 
-------------------------------------------

leg = Legend {
	grouping = "uniquevalue",
	colorBar = {
		{value = EMPTY, color = "white"},
		{value = FULL, color = "black"},
		{value = WOOD, color = "brown"},
		
	}
}

Observer {
	subject = cs,
	attributes = {"state"},
	legends = {leg}
}


e:execute(600)